import { lazy, Suspense } from 'react';
import { BrowserRouter, Switch } from 'react-router-dom';
import { connect } from 'react-redux';
import { ToastContainer } from 'react-toastify';

import s from './App.module.css';
import { IAppProps, IAppState } from './typescript/components/App/interfaces';

import PrivateRoute from './components/Routes/PrivateRoute/PrivateRoute';
import PublicRoute from './components/Routes/PublicRoute/PublicRoute';
import Loader from './components/Loader/Loader';

const Navigation = lazy(
  () =>
    import(
      './components/Navigation/Navigation' /* webpackChunkName: "Navigation" */
    ),
);

const Categories = lazy(
  () =>
    import(
      './components/Categories/Categories' /* webpackChunkName: "Categories" */
    ),
);

const Goods = lazy(
  () => import('./components/Goods/Goods' /* webpackChunkName: "Goods" */),
);

const DetailGood = lazy(
  () =>
    import(
      './components/Goods/DetailGood/DetailGood' /* webpackChunkName: "DetailGood" */
    ),
);

const Orders = lazy(
  () => import('./components/Orders/Orders' /* webpackChunkName: "Orders" */),
);

const DetailOrder = lazy(
  () =>
    import(
      './components/Orders/DetailOrder/DetailOrder' /* webpackChunkName: "DetailOrder" */
    ),
);

const Authorization = lazy(
  () =>
    import(
      './components/Authorization/Authorization' /* webpackChunkName: "AuthorizationPage" */
    ),
);

const SignInForm = lazy(
  () =>
    import(
      './components/Authorization/SignIn/SignIn' /* webpackChunkName: "SignInForm" */
    ),
);

const RegistrationForm = lazy(
  () =>
    import(
      './components/Authorization/Registration/RegistrationForm' /* webpackChunkName: "RegistrationForm" */
    ),
);

const UserInterface = lazy(
  () =>
    import(
      './components/UserInterface/UserInterface' /* webpackChunkName: "UserInterface" */
    ),
);

function App({ isLoading }: IAppProps) {
  return (
    <div className={s.appWrapper}>
      <Suspense fallback={<Loader />}>
        <BrowserRouter>
          <Switch>
            <PrivateRoute exact path="/">
              <Navigation />
              <UserInterface />
            </PrivateRoute>
            <PrivateRoute exact path="/categories">
              <Navigation />
              <UserInterface />
              <Categories />
            </PrivateRoute>
            <PrivateRoute exact path="/categories/goods/:name">
              <Navigation />
              <UserInterface />
              <Goods />
            </PrivateRoute>
            <PrivateRoute exact path="/categories/goods/:name/:id">
              <Navigation />
              <UserInterface />
              <DetailGood />
            </PrivateRoute>
            <PrivateRoute exact path="/orders">
              <Navigation />
              <UserInterface />
              <Orders />
            </PrivateRoute>
            <PrivateRoute exact path="/orders/:id">
              <Navigation />
              <UserInterface />
              <DetailOrder />
            </PrivateRoute>
            <PublicRoute path={'/authorization'} restricted>
              <Authorization />
              <PublicRoute exact path={'/authorization/login'} restricted>
                <SignInForm />
              </PublicRoute>
              <PublicRoute
                exact
                path={'/authorization/registration'}
                restricted
              >
                <RegistrationForm />
              </PublicRoute>
            </PublicRoute>
          </Switch>
        </BrowserRouter>
      </Suspense>
      {isLoading && <Loader />}
      <ToastContainer
        position="top-right"
        autoClose={3000}
        hideProgressBar={false}
        newestOnTop={false}
        closeOnClick
        rtl={false}
        pauseOnFocusLoss
        draggable
        pauseOnHover
      />
    </div>
  );
}

const mapStateToProps = (state: IAppState) => ({
  isLoading: state.isLoading.flag,
});

export default connect(mapStateToProps, null)(App);
